/* cursed.c - partial (dumb) replacement for curses */

/* Written 1993 by Werner Almesberger */


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <termios.h>
#include <termcap.h>
#include <sys/ioctl.h>

#include "common.h"
#include "cursed.h"


int LINES = 24;
int COLS = 80;


static struct termios original,current;
static char *cl,*ce,*so,*se,*cm;
static char termcap_area[1024*5];
static int tty;


void clear(void)
{
    printf("%s",cl);
}


void clrtoeol(void)
{
    printf("%s",ce);
}


void standout(void)
{
    printf("%s",so);
}


void standend(void)
{
    printf("%s",se);
}


void move(int y,int x)
{
    printf("%s",tgoto(cm,x,y));
}


void refresh(void)
{
    fflush(stdout);
}


void initscr(void)
{
    struct winsize winsz;
    static char bp[1024];
    char *term,*area;

    if (!(term = getenv("TERM")) || tgetent(bp,term) <= 0)
	die("No termcap entry");
    if ((LINES = tgetnum((char *) "li")) < 0) LINES = 24;
    if ((COLS = tgetnum((char *) "co")) < 0) COLS = 80;
    if (ioctl(1,TIOCGWINSZ,&winsz) >= 0) {
	if (winsz.ws_col) COLS = winsz.ws_col;
	if (winsz.ws_row) LINES = winsz.ws_row;
    }
    area = termcap_area;
    if (!(cl = tgetstr((char *) "cl",&area)))
	die("Don't know how to clear screen");
    if (!(ce = tgetstr((char *) "ce",&area)))
	die("Don't know how to clear to EOL");
    if (!(so = tgetstr((char *) "so",&area)))
	die("Don't know how to begin standout");
    if (!(se = tgetstr((char *) "se",&area)))
	die("Don't know how to end standout");
    if (!(cm = tgetstr((char *) "cm",&area)))
	die("Don't know how to move cursor");
    tty = tcgetattr(0,&original) >= 0;
    current = original;
}


void noecho(void)
{
    current.c_lflag &= ~ECHO;
    if (tty) (void) tcsetattr(0,TCSADRAIN,&current);
}


void raw(void)
{
    current.c_lflag &= ~(ICANON | ISIG);
    if (tty) (void) tcsetattr(0,TCSADRAIN,&current);
}


void echo(void)
{
    current.c_lflag |= ECHO;
    if (tty) (void) tcsetattr(0,TCSADRAIN,&current);
}


void noraw(void)
{
    current.c_lflag |= ICANON | ISIG;
    if (tty) (void) tcsetattr(0,TCSADRAIN,&current);
}


void endwin(void)
{
    if (tty) (void) tcsetattr(0,TCSADRAIN,&original);
}


int getch(void)
{
    int size;
    char ch;

    while ((size = read(0,&ch,1)) <= 0)
	if (size < 0 && errno != EINTR) return EOF;
    return ch;
}
