/* descr.h - configuration description */

/* Written 1993 by Werner Almesberger */


#ifndef DESCR_H_
#define DESCR_H_


typedef struct {
    const char *long_name;
    const char *short_name; /* NULL if == long_name */
    const char *value; /* NULL if == long_name */
    int expand;
    struct _item *entries; /* NULL is no private section */
} VALUE_ENTRY;

typedef enum { it_title,it_yesno,it_number,it_choice,it_set } ITEM_TYPE;

typedef struct _item {
    ITEM_TYPE type;
    const char *name; /* NULL if it_title */
    const char *long_name;
    const char *short_name; /* NULL if == long_name */
    const char *file; /* NULL if none */
    int written; /* set during output creation if item has been saved */
    int deflt; /* bool: non-zero means "true", else index or set */
    int value; /* -1 = none, else like "default" */
    int active; /* active variable if it_set */
    VALUE_ENTRY *values; /* NULL if title, yesno or number */
    int num_values; /* if it_set or it_choice */
    int low,high; /* limits if it_number */
    int expand;
    struct _item *parent;
    struct _item *entries; /* must be NULL if it doesn't apply */
    struct _item *prev,*next;
} ITEM;

typedef enum {
  et_or,et_xor,et_and,et_not,				/* boolean */
  et_plus,et_minus,et_times,et_div,et_mod,et_uminus,	/* arithmetic */
  et_eq,et_ne,et_lt,et_gt,et_le,et_ge,			/* relational */
  et_atom,et_number					/* values */
} EXPR_TYPE;

typedef struct _expr {
    EXPR_TYPE type;
    union {
	struct {
	    const struct _expr *left,*right;
	} op;
	struct {
	    const ITEM *var;
	    int value;
	} atom;
	struct {
	    int value;
	} number;
    } u;
} EXPR;

typedef struct _error {
    const char *msg;
    const EXPR *expr;
    int ignore;
    struct _error *next;
} ERROR;


extern ITEM *descr; /* configuration description tree */


#define VALUE_OF(c) ((c)->value == -1 ? (c)->deflt : (c)->value)
#define IS_VARIABLE(c) ((c)->type != it_title)
#define IS_LISTABLE(c) ((c)->type != it_title && (c)->type != it_number)
#define EXPAND(c) (*((c)->type == it_choice ? \
  &(c)->values[VALUE_OF(curr)].expand : \
  (c)->type == it_set ? &(c)->values[curr->active].expand : \
  &(c)->expand))


const char *dsc_error(void);

/* Performs all consistency checks defined in the current description. Returns
   NULL if the configuration is value, a pointer to an error message if an
   error has been detected. Only one error can be found at a time. */

int dsc_ignore(void);

/* Ignore the previously issued error. */

void dsc_age(void);

/* Age the "current error". Each issued error has an initial time to live (TTL)
   of two. If the error's TTL has reached zero, further DSC_IGNOREs have no
   effect. */

void dsc_load(const char *name);

/* Opens the specified file and loads the configuration description contained
   in it. Terminates the program if any error occurs. */

#endif
