/* file.c - output file handling */

/* Written 1993 by Werner Almesberger */


#ifndef FILE_H_
#define FILE_H_


#include <stdio.h>


typedef struct {
    const char *templ_name;
    const char *file_name;
    const char *tmp_name;
    FILE *templ,*tmp;
} FILE_DSC;


extern char file_error[]; /* *file_error is an error has occurred */


void file_reset(void);

/* Initializes the file creation subsystem and clears the error message. */

void file_open(FILE_DSC *dsc,const char *name);

/* Opens the specified file for writing. An optional template file is copied
   to the output file. FILE_OPEN creates a temporary output file which can
   later (FILE_DONE) be renamed to the real target file. The FILE_* routines
   report errors by writing them into FILE_ERROR, but continue to work. */

const char *file_name(FILE_DSC *dsc);

/* Returns the name of the target file of the specified descriptor. */

void file_printf(FILE_DSC *dsc,const char *fmt,...);

/* Writes a printf-style string to the file. */

void file_token(FILE_DSC *dsc,const char *token);

/* Writes a token to the file. The quoting conventions of kconf's scanner are
   observed. */

void file_close(FILE_DSC *dsc);

/* Closes the files associated with the specified descriptor. The temporary
   file is not yet renamed. This way, renaming can be deferred until all files
   have been successfully closed. */

void file_done(FILE_DSC *dsc,int commit);

/* Renames the temporary file associated with the descriptor if commit is
   non-zero. Otherwise, the temporary file is erased. */

#endif
