/* kconf.c - Linux kernel configurator */

/* Written 1993 by Werner Almesberger */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "common.h"
#include "values.h"
#include "descr.h"
#include "tree.h"
#include "kconf.h"


const char *config_output = "config.out";
const char *make_output = "make.out";
const char *default_output = NULL;
int no_new = 0;


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [ -l ] [ -n ] [ -i input_config ] "
      "[ -o output_config ]\n",name);
    fprintf(stderr,"%13s[ -a default_output ] [ -m make_output ] "
      "config_description ...\n","");
    exit(1);
}


static void list_errors(const char *name)
{
    FILE *file;
    const char *error;

    if ((file = fopen(name,"w")) == NULL) {
	fprintf(stderr,"create %s: %s\n",name,strerror(errno));
	exit(1);
    }
    while (error = dsc_error()) {
	if (fprintf(file,"%s\n",error) == EOF) {
	    fprintf(stderr,"write %s: %s\n",name,strerror(errno));
	    exit(1);
	}
	dsc_ignore();
    }
    if (fclose(file) == EOF) {
	fprintf(stderr,"close %s: %s\n",name,strerror(errno));
	exit(1);
    }
}


int main(int argc,char **argv)
{
    const char *name,*config_input,*error_list;
    int line;

    config_input = error_list = NULL;
    line = 0;
    name = *argv++;
    argc--;
    while (argc && **argv == '-') {
	argc--;
	if (argv[0][2]) usage(name);
	switch ((*argv++)[1]) {
	    case 'l':
		if (line) usage(name);
		line = 1;
		break;
	    case 'n':
		if (no_new) usage(name);
		no_new = 1;
		break;
	    case 'i':
		if (!argc || config_input) usage(name);
		config_input = *argv++;
		argc--;
		break;
	    case 'm':
		if (!argc) usage(name);
		make_output = *argv++;
		argc--;
		break;
	    case 'o':
		if (!argc) usage(name);
		config_output = *argv++;
		argc--;
		break;
	    case 'a':
		if (!argc || default_output) usage(name);
		default_output = *argv++;
		argc--;
		break;
	    case 'E': /* undocumented, used by regression test */
		if (!argc || error_list) usage(name);
		error_list = *argv++;
		argc--;
		break;
	    default:
		usage(name);
	}
    }
    if (!argc) usage(name);
    while (argc--) dsc_load(*argv++);
    if (config_input) val_load(config_input);
    if (line) line_mode(descr);
    else full_screen(descr);
    if (error_list) list_errors(error_list);
    return 0;
}
