/* scanner.c - configuration database scanner */

/* Written 1993 by Werner Almesberger */


#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "common.h"
#include "scanner.h"


#define MAX_TOKEN 300


char token;
char string[MAX_TOKEN+1];
int scan_debug = 0;

static FILE *file = NULL;
static const char *file_name;
static int line,returned;


void scan_error(const char *msg,...)
{
    va_list ap;

    va_start(ap,msg);
    fprintf(stderr,"%s %d: ",file_name,line);
    vfprintf(stderr,msg,ap);
    va_end(ap);
    fputc('\n',stderr);
    exit(1);
}


void scan_open(const char *name)
{
    if (file) scan_error("File \"%s\" is still open",file_name);
    if (!(file = fopen(file_name = name,"r"))) {
	perror(name);
	exit(1);
    }
    returned = 0;
    line = 1;
}


void scan_close(void)
{
    (void) fclose(file);
    file = NULL;
}


static int do_scan_next(void)
{
    char *here;
    int ch,quote;

    if (returned) {
	returned = 0;
	return 1;
    }
    while (1) {
	while ((ch = getc(file)), ch == ' ' || ch == '\t' || ch == '\n')
	    if (ch == '\n') line++;
	if (ch != '#')
	    if (ch == EOF) return 0;
	    else break;
	else {
	    while ((ch = getc(file)), ch != EOF && ch != '\n');
	    line++;
	}
	if (ch == EOF) return 0;
    }
    token = 0;
    here = string;
    if (ch == '"' || ch == '\'' ) {
	quote = ch;
	while ((ch = getc(file)), ch != EOF && ch != '\n') {
	    if (ch == quote) {
		*here = 0;
		return 1;
	    }
	    if (ch == '\\') {
		ch = getc(file);
		if (ch == EOF || ch == ' ' || ch == '\t' || ch == '\n')
		    scan_error("improper use of \\");
	    }
	    *here++ = ch;
	}
	scan_error("unterminated string");
    }
    if (!isalnum(ch) && ch != '_') {
	token = ch;
	return 1;
    }
    do {
	if (!isalnum(ch) && ch != '_') {
	    ungetc(ch,file);
	    break;
	}
	*here++ = ch;
    }
    while ((ch = getc(file)) != EOF);
    *here = 0;
    return 1;
}


int scan_next(void)
{
    int ok;

    ok = do_scan_next();
    if (scan_debug)
	if (!ok) printf("EOF\n");
	else if (token) printf("TOKEN \"%c\"\n",token);
	    else printf("STRING \"%s\"\n",string);
    return ok;
}


void scan_real(void)
{
    if (!scan_next()) scan_error("unexpected EOF");
}


void scan_string(void)
{
    scan_real();
    if (token) scan_error("string expected");
}


void scan_return(void)
{
    if (scan_debug) printf("RETURN\n");
    returned = 1;
}


int scan_quote(const char *str)
{
    while (*str)
	if (!isalnum(*str) && *str != '_') return 1;
	else str++;
    return 0;
}
