/* scanner.h - configuration database scanner */

/* Written 1993 by Werner Almesberger */


#ifndef SCANNER_H_
#define SCANNER_H_

extern char token; /* the last token or 0 if a string has been scanned */
extern char string[]; /* string if !TOKEN */
extern int scan_debug; /* set to non-zero to enable debug messages */

void scan_open(const char *name);

/* Opens the specified file for scanning. Terminates the program on error. */

void scan_close(void);

/* Closes the scanned file. */

void scan_error(const char *msg,...);

/* Reports a scanner or parser error and terminates the program. MSG is a
   printf-style message. */

int scan_next(void);

/* Fetches the next token and stores it in the variables TOKEN and STRING.
   Returns zero if at EOF or a non-zero integer. */

void scan_real(void);

/* Like SCAN_NEXT, but encountering EOF is a fatal error. */

void scan_string(void);

/* Like SCAN_REAL, but encountering non-strings is a fatal error. */

void scan_return(void);

/* Returns the last token. */

int scan_quote(const char *str);

/* Returns a non-zero integer if STR has to be quoted in order to appear as one
   token to the scanner. */

#endif
