/* tree.h - configuration data tree access */

/* Written 1993 by Werner Almesberger */


#ifndef TREE_H_
#define TREE_H_

#include "descr.h"


int tree_num_values(const ITEM *curr);

/* Returns the number of values associated with the specified item. Passing a
   pointer to a value-less item is a fatal error. */

const char *tree_nth_value(const ITEM *curr,int number);

/* Returns the NUMBERth value of the specified item. Values are counted
   starting with zero. Specifying a value-less item is a fatal error. */

const char *tree_value(const ITEM *curr);

/* Returns the current value of the specified item. TREE_VALUE can only be used
   with items that have exactly one value, i.e. IT_YESNO and IT_CHOICE. */

int tree_expanded(const ITEM *curr);

/* Returns a non-zero integer if the specified item is expanded, i.e. if a list
   of sub-items should be displayed, if such a list exists. */

ITEM *tree_section(const ITEM *curr);

/* Returns a pointer to the first item of the active section of CURR. A section
   is active if either the variable is set (IT_YESNO), the corresponding value
   is selected (IT_CHOICE) or the value is active (IT_SECTION). Visibility as
   controlled by the EXPAND flag does not matter. */

ITEM *tree_entries(const ITEM *curr);

/* Like TREE_SECTION, but also sections of disabled items are returned. */

int tree_prev(ITEM **curr);

/* Moves CURR one item up. "up" is defined as the item preceding the current
   one when the tree is flattened. Returns a non-zero integer if CURR has been
   changed, zero if *CURR already points to the first item. */

int tree_next(ITEM **curr);

/* Moves CURR one item down. "down" is defined as the item following the
   current one when the tree is flattened. Returns a non-zero integer if CURR
   has been changed, zero if *CURR already points to the last item. */

int tree_seq(ITEM **curr);

/* Like TREE_NEXT, but iterates through all values of sets. This is used in
   line mode. */

int tree_level(const ITEM *curr);

/* Returns the number of the level at which the item CURR is located. The root
   of the tree is defined to be at level 0. */

int tree_new(const ITEM *curr);

/* Returns a non-zero integer if the item CURR points to is not defined by the
   old configuration settings or if any enabled item below it has this
   property. */

const char *tree_line(const ITEM *curr,int max_len);

/* Returns a pointer to a static area which contains a line describing the item
   CURR points to. The description consists of the long name of CURR and
   possibly a summary of the items below it. The length of the line is limited
   to MAX_LEN characters. The a full summary would exceed the available space,
   descriptions of the lowest items are hidden. */

void tree_change(ITEM *curr,int next);

/* Changes the value of CURR either to the next (if NEXT is non-zero) or to the
   previous (if NEXT is zero) value. If CURR points to a set item, only the
   active marker is moved, but the value is not changed. */

void tree_toggle(ITEM *curr);

/* Toggles the active value of the specified item. Has not effect of CURR
   doesn't point to a set item. */

void tree_set_number(ITEM *curr,int number);

/* Sets the specified numeric variable to NUMBER. No bound checking is done. */

#endif
